import os
import inspect
import syslog
import traceback


def _get_final_entry_text(entry_level, entry_text):
    """
    :type entry_level: str
    :type entry_text: str
    :rtype: str
    """
    template = '[{entry_level}|{caller_file}:{caller_line}|{caller_class}::{caller_fn}] {entry_text}'

    caller_stack_frame = inspect.stack()[2][0]
    caller_file = os.path.basename(inspect.getfile(caller_stack_frame))
    caller_line = inspect.getlineno(caller_stack_frame)
    caller_fn = inspect.stack()[2][3]
    try:
        caller_class = caller_stack_frame.f_locals["self"].__class__.__name__
    except KeyError:
        template = '[{entry_level}|{caller_file}:{caller_line}|{caller_fn}] {entry_text}'

    return template.format(**locals())


def open_log(module_ident, log_level='info'):
    """
    :type module_ident: str
    :type log_level: str
    """
    edge_to_syslog_log_level_map = {'trace': syslog.LOG_DEBUG,
                                    'debug': syslog.LOG_DEBUG,
                                    'info': syslog.LOG_INFO,
                                    'warn': syslog.LOG_WARNING,
                                    'error': syslog.LOG_ERR,
                                    'none': syslog.LOG_CRIT}

    syslog.openlog(ident=module_ident, logoption=syslog.LOG_PID, facility=syslog.LOG_DAEMON)
    syslog_log_level = edge_to_syslog_log_level_map[log_level]
    priority_mask = syslog.LOG_UPTO(syslog_log_level)
    syslog.setlogmask(priority_mask)


def close_log():
    syslog.closelog()


def log_debug(entry_text):
    """
    :type entry_text: str
    """
    syslog.syslog(syslog.LOG_DEBUG, _get_final_entry_text('d', entry_text))


def log_info(entry_text):
    """
    :type entry_text: str
    """
    syslog.syslog(syslog.LOG_INFO, '[i] {}'.format(entry_text))


def log_warning(entry_text):
    """
    :type entry_text: str
    """
    syslog.syslog(syslog.LOG_WARNING, '[w] {}'.format(entry_text))


def log_error(entry_text):
    """
    :type entry_text: str
    """
    syslog.syslog(syslog.LOG_ERR, '[e] {}'.format(entry_text))


def log_debug_last_exc():
    log_entry_set = traceback.format_exc().split('\n')
    for log_entry in log_entry_set:
        syslog.syslog(syslog.LOG_DEBUG, _get_final_entry_text('d', log_entry))
