import os
import signal
import shutil
import subprocess
from subprocess import check_output
from os.path import dirname as path_dirname, exists as path_exists

from path_helpers import *
import eventlog as eventlog
from edge.fs_helpers import create_jnode_from_default

_UI_VIDEOSOUCE_JNODE_DIRPATH = '/etc/edge/setupstore/admin/actions/pipeline/%s/videosource'
_UI_PIPELINE_SET_FILEPATH = '/etc/edge/setupstore/admin/actions/pipeline.json'
_WS_RESTART_FILEPATH = '/var/spool/edge/ws0-ws0-restart'
_SS_RESTART_FILEPATH = '/var/spool/edge/ss0-ss0-restart'
PIPELINE_SET_FILEPATH = '/etc/edge/setupstore/admin/actions/pipeline.json'
PIPELINE_DEFSET_FILEPATH = '/opt/edge/share/setupstore/ui/pipeline-jnode.json'


def restart_edge_module(edge_type, edge_id):
    """
    :type edge_type: str
    :type edge_id: str
    """
    eventlog.log_debug('restarting edge module %s id %s' % (edge_type, edge_id))
    subprocess.Popen(["kill -9 `ps ax |"
                      "grep \"%s --id.%s\" |"
                      "grep -v grep |"
                      "awk '{print $1}'`" % (edge_type, edge_id)],
                     shell=True)


def sighup_edge_module(edge_type, edge_id):
    """
    :type edge_type: str
    :type edge_id: str
    """
    eventlog.log_debug('sighup edge module %s id %s' % (edge_type, edge_id))
    subprocess.Popen(["kill -1 `ps ax |"
                      "grep \"%s --id.%s\" |"
                      "grep -v grep |"
                      "awk '{print $1}'`" % (edge_type, edge_id)],
                     shell=True)


def restart_webserver_lazy():
    eventlog.log_debug('restarting webserver')
    # PCD restarts webserver when this file appears
    silent_mkdir_r(path_dirname(_WS_RESTART_FILEPATH))
    ws_restart_filepath = _WS_RESTART_FILEPATH

    with open(ws_restart_filepath, 'w'):
        pass


def load_db_json():
    """
    :rtype: dict | None
    """
    db_conf = load_json_file(DB_JSON_FILEPATH, utf8=True)
    return db_conf


def get_post_guid_name():
    """
    :rtype: (str, str) | None
    """
    db_conf = load_db_json()
    if not db_conf:
        return

    post_jnode = db_conf.get('post')
    if not post_jnode:
        return

    post_guid = post_jnode.get('uuid')
    post_name = post_jnode.get('name')

    return post_guid, post_name


def set_post_name(post_name):
    """
    :type post_name: str
    """
    db_conf = load_json_file(DB_JSON_FILEPATH)
    if db_conf:
        post_jnode = db_conf.get('post')
        if post_jnode:
            post_jnode['name'] = post_name

        else:
            db_conf['post'] = {
                'name': post_name
            }
    else:
        db_conf = {
            'post': {
                'name': post_name
            }
        }

    with open(DB_JSON_FILEPATH, 'w') as json_file:
        json.dump(db_conf, json_file, indent=2)


def get_edge_group(ui_pid):
    """
    :type ui_pid: str
    :rtype: str
    """
    return str(int(ui_pid) * 10)


def get_edge_id(edge_group, edge_identity):
    """
    :type edge_group: str
    :type edge_identity: EdgeIdentity
    :rtype: str
    """
    if edge_identity.type in [EDGE_TYPE_WEBSERVER, EDGE_TYPE_SETUPSTORE]:
        return '0'

    return str(int(edge_group) + int(edge_identity.gid))


def restart_edge_modules(edge_type):
    """
    :type edge_type: str
    """
    eventlog.log_debug('restarting all edge modules %s' % edge_type)
    subprocess.Popen(["pkill -9 %s" % edge_type], shell=True)


def restart_edge_server():
    # PCD restarts edgeserver when this file appears
    with open(_SS_RESTART_FILEPATH, 'w'):
        pass


def sighup_setupstore():
    eventlog.log_debug('restarting setupstore')
    os.kill(int(check_output(['pidof', 'setupstore'])), signal.SIGHUP)


def load_json_file(json_filepath, utf8=False):
    """
    :type json_filepath: str
    :type utf8: bool
    :rtype: dict | list  | None
    """
    jnode = None
    try:
        encoding = 'utf-8' if utf8 else None
        with open(json_filepath) as json_file:
            jnode = json.load(json_file, encoding=encoding)

    except IOError as x:
        eventlog.log_debug("fail: read %s (%s, ERRNO: %s)" % (json_filepath, x.message, x.errno))

    except ValueError as x:
        eventlog.log_debug("fail: deserialize %s (%s)" % (json_filepath, x.message))

    return jnode


def load_json_file2(json_filepath):
    """
    :type json_filepath: str
    :rtype: dict | list
    :raise IOError:
    :raise ValueError:
    """
    try:
        with open(json_filepath) as json_file:
            jnode = json.load(json_file)

    except IOError as x:
        eventlog.log_debug("fail: read %s (%s, ERRNO: %s)" % (json_filepath, x.message, x.errno))
        raise

    except ValueError as x:
        eventlog.log_debug("fail: deserialize %s (%s)" % (json_filepath, x.message))
        raise

    return jnode


def silent_mkdir_r(target_path):
    """
    :type target_path: str
    """
    if not target_path:
        return

    try:
        os.makedirs(target_path)

    except (OSError, TypeError) as x:
        eventlog.log_debug("fail: make %s (%s)" % (target_path, x.message))


def silent_rm_r(target_path):
    """
    :type target_path: str
    """
    if not target_path:
        return

    try:
        # try remove file
        try:
            os.remove(target_path)
            return
        except EnvironmentError:
            pass

        # try remove nested-files in directory
        shutil.rmtree(target_path, ignore_errors=True)

    except EnvironmentError as x:
        eventlog.log_debug("fail: remove %s (%s)" % (target_path, x.message))


def get_videsource_status(ui_pid, ui_id):
    """
    :type ui_pid: str
    :type ui_id: str
    :rtype: int | None
    """
    ui_videosource_jnode_path = join_path(_UI_VIDEOSOUCE_JNODE_DIRPATH % ui_pid, "0.json")
    try:
        with open(ui_videosource_jnode_path) as json_file:
            ui_jnode = json.load(json_file)

    except IOError as x:
        eventlog.log_debug('fail: load ui-vs-jnode %s (%s, Errno: %s)' %
                           (ui_videosource_jnode_path, x.message, x.errno))
        return None

    statfile_dirpath = ui_jnode['ro']['stats-dirpath']
    statfile_path = join_path(statfile_dirpath, 'pipeline-%s/zabbix' % ui_pid, 'vs%s' % ui_id)

    try:
        with open(statfile_path, 'r') as statfile:
            statfile_lines = statfile.readlines()

    except IOError as x:
        eventlog.log_debug('no stat-file %s (%s, Errno: %s)' % (statfile_path, x.message, x.errno))
        return None

    try:
        stat_params = {}
        for line in statfile_lines:
            key, value = line.split(':', 1)
            stat_params[key.strip()] = value.strip()

    except ValueError as x:
        eventlog.log_debug('fail: parse stat-file %s (%s)' % (statfile_path, x.message))
        return None

    return int(stat_params['error'])


def _get_rtsp_videosource_image_wh(rtsp_videosource_ui_jnode, ui_pid, ui_id):
    """
    :type rtsp_videosource_ui_jnode: dict
    :type ui_pid: str
    :type ui_id: str
    :rtype: (int, int) | None
    """
    statfile_dirpath = rtsp_videosource_ui_jnode['ro']['stats-dirpath']
    statfile_path = join_path(statfile_dirpath, 'pipeline-%s/zabbix' % ui_pid, 'vs%s' % ui_id)

    try:
        with open(statfile_path, 'r') as statfile:
            statfile_lines = statfile.readlines()

    except IOError as x:
        eventlog.log_debug('no rtsp-stat-file %s (%s, Errno: %s)' % (statfile_path, x.message, x.errno))
        return None

    try:
        stat_params = {}
        for line in statfile_lines:
            key, value = line.split(':', 1)
            stat_params[key.strip()] = value.strip()

    except ValueError:
        eventlog.log_debug('fail: parse stat-file %s' % statfile_path)
        return None

    if stat_params['error'] != '0':
        eventlog.log_debug('stat-file error: %s' % stat_params['error'])

    try:
        image_width = stat_params['image-width']
        image_height = stat_params['image-height']
    except KeyError:
        eventlog.log_debug('fail: parse stat-file %s' % statfile_path)
        return None

    return int(image_width), int(image_height)


def get_videosource_image_wh(ui_pid, ui_id, ui_videosourse_jnode=None):
    """
    :type ui_pid: str
    :type ui_id: str
    :type ui_videosourse_jnode: dict | None
    :rtype: (int, int) | None
    """
    if ui_videosourse_jnode is None:
        ui_jnode_filepath = get_ui_pipeline_jnode_filepath(ui_pid, UI_TYPE_VIDEOSOURCE, ui_id)
        eventlog.log_debug("load: %s" % ui_jnode_filepath)
        videosource_jnode = load_json_file(ui_jnode_filepath)
    else:
        videosource_jnode = ui_videosourse_jnode

    if videosource_jnode is not None:
        if videosource_jnode['rw']['type'] == 'rtsp':
            return _get_rtsp_videosource_image_wh(videosource_jnode, ui_pid, ui_id)
        else:
            resolution_jnode = videosource_jnode['rw']['videostream']['resolution']
            return int(resolution_jnode['w']), int(resolution_jnode['h'])
    else:
        return None


def get_pipeline_jnode_set():
    """
    :rtype: list[dict] | None
    """
    # create ui-jnode
    pipeline_jnode_set = get_pipeline_jnode_set2()
    if pipeline_jnode_set is not None:
        for pipeline in pipeline_jnode_set:
            pipeline.pop('type', None)

        pipeline_jnode_set.sort(key=lambda x: x['id'])
        return pipeline_jnode_set

    return None


def get_pipeline_jnode_set2():
    """
    :rtype: list[dict] | None
    """
    # create ui-jnode
    if not path_exists(PIPELINE_SET_FILEPATH):
        # noinspection PyBroadException
        try:
            create_jnode_from_default(
                PIPELINE_DEFSET_FILEPATH,
                PIPELINE_SET_FILEPATH)

        except Exception:
            eventlog.log_debug_last_exc()
            return None

    with open(PIPELINE_SET_FILEPATH, 'r') as json_file:
        origins = json.load(json_file)

    return origins


def load_pscuctl_json():
    """
    :rtype: dict | None
    """
    psuctl_conf = load_json_file(PSUCTL_JSON_FILEPATH, utf8=True)
    return psuctl_conf


def get_psuctl_endpoint():
    """
    :rtype: str | None
    """
    psuctl_conf = load_pscuctl_json()
    if not psuctl_conf:
        eventlog.log_debug('fail: psu-ctl.conf not found')
        return

    ro_jnode = psuctl_conf.get('ro')
    if not ro_jnode:
        eventlog.log_debug('fail: param psu-ctl.conf#/ro has invalid value')
        return

    com_port = ro_jnode.get('com-port')
    if not com_port:
        eventlog.log_debug('fail: param psu-ctl.conf#/ro/com-port has invalid value')
        return

    return com_port